# R script to analyze the experimental data by Scheibehenne & Studer

rm(list=ls(all=TRUE))
graphics.off()
library(runjags)

setwd("C:/") #set working directory 

source("runBUGS.r")

#load raw data from the experiment
load("rawData.RData")
rawDataList=dataList$rawDataList
pStayMatrix=dataList$pStayMatrix
stayMatrix =dataList$stayMatrix
nMatrix    =dataList$nMatrix
seqMatrix  =dataList$seqMatrix
nChoices   =dim(rawDataList[[1]])[1]
nVpn       =length(rawDataList)
 
 
# some handy functions...

	#bootstrapping of a vector
	bootstrap = function (vec, ci=.95, nsamples=10000) {

	#re-sample vector
	sampleMean=rep(NA, times=nsamples)
	for (ss in 1:nsamples) {
		sampleMean[ss]=mean(sample(vec, replace=T), na.rm=T)
	}
	ci=quantile(sampleMean, probs=c(1-ci,ci), na.rm=T)
	
	return(ci)
	}	

	#bootstrap correlation
	corrBootstrap = function (xx, yy, ci=.95, nStraps=10000) {
		corrVec=rep(NA, times=nStraps)
		for (ss in 1:nStraps) {
			this.Sample=sample.int(n=length(xx), replace=T)
			corrVec[ss] = cor(xx[this.Sample], yy[this.Sample])
		}
		ciCorr=quantile(corrVec, probs=c(1-ci,ci), na.rm=T)
	
		return(ciCorr)
	}
	

#----------------------	
#descriptive statistics

#time to complete experiment
	timeInMinutes=sapply(rawDataList, function(i) max(i$StartTime)/60000)
	mean(timeInMinutes)
	sd(timeInMinutes)
	
#average number of correct predictions
	pCorrect=sapply(rawDataList, function(i) sum(i$Outcome>0)/256)
	round(mean(pCorrect)*100,1)
	round(sd(pCorrect)*100,1)

#average payoff
	 punkte=sapply(rawDataList, function(i) sum(i$Outcome))
	 initialEndowment=500
	 endpoints=punkte+initialEndowment
	 mean(endpoints)
	 sd(endpoints)
	
	
#probability to stay
	windows()
	meanMatrix=rowMeans(pStayMatrix, na.rm=T)
	ci95Table=apply(pStayMatrix, 1,bootstrap)
	xpos=barplot(meanMatrix, ylim=c(0,1), xlab="Sequence length", ylab="p(stay)", main="Mean Probability of Staying on the Same Symbol")
	abline(h=.5, lty="dotted")
	arrows(xpos, ci95Table[1,], xpos, ci95Table[2,], code = 3,angle=90, length=.05)
	
	

nStayMatrix=pStayMatrix*nMatrix


#================================
# Model comparison on the individual level
	
BUGSmodel="modelComparisonIndividual.txt"

#BUGS parameters
	nBurnin    =5000
    chainLength=25000
	thinSteps  =100
	nChains    =4
    parameters =c("idx")
	initial.pDriftModel=.5
  
  #initialize starting values for the chains
  initsFunc <-function () {
   list (
	   intercept0Phi = runif(1,-2, 2),
	   slope01Phi    = runif(1,-2,2), 
	   intercept1Phi = runif(1,-2,2),
	   idx           = sample(c(1,2),1)
       )
  }
  
 

#run BUGS model for each individual
meanIdx=rep(NA, times=nVpn)
pSwitch=rep(NA, times=nVpn)


for (vpn in 1:nVpn) {

  #extract individual data
  dataList = list(
				nStay       = nStayMatrix[,vpn],
				nTotal      = nMatrix[,vpn],
				pDriftModel = initial.pDriftModel
				)

  #send it off to BUGS
	samplesComparison=runBUGS(model=BUGSmodel,dataList, initsFunc, parameters,nBurnin, chainLength, thinSteps,nChains) 
	mcmcChains = as.matrix(samplesComparison$mcmc)
	
	#obtain idx
	idx = mcmcChains[,"idx"]
	
	#calculate pSwitch
	nSamples=length(idx)
	pSwitch[vpn]=sum(idx[1:(nSamples-1)]!=idx[2:nSamples])/nSamples
	meanIdx[vpn]=mean(idx)-1 #probability of simple model (idx=2) over drift model (idx=1)
}	

#calculate BF
	post.pDriftModel=1-meanIdx
	priorBF=(initial.pDriftModel/(1-initial.pDriftModel))
	postBF=post.pDriftModel/(1-post.pDriftModel)
	BF=priorBF*postBF
	logBF=log(BF)
   
  
#check efficiency
	windows()
	plot(pSwitch, logBF, pch=20, cex=1.3)
	abline(h=0)
	

	#barplot with BF categories
	
		#1-3 anecdotal
		#3-10 substantial
		#10-30 strong
		#30-100 very strong
		#> 100 decisive
	cutoff=c(1,3,10,30,100, Inf)
	
	BFtable=matrix(0, nrow=2, ncol=5)
	colnames(BFtable)=c("BF 1-3", "BF 3-10", "BF 10-30", "BF 30-100", "BF >100")
	rownames(BFtable)=c("Stationary model","Drift model")
	for(vpn in 1:nVpn) {
		for(cc in 2:length(cutoff)) {
			if(BF[vpn]>cutoff[cc-1] & BF[vpn]<cutoff[cc]) BFtable["Drift model",cc-1]=BFtable["Drift model",cc-1]+1
			if(1/BF[vpn]>cutoff[cc-1] & 1/BF[vpn]<cutoff[cc]) BFtable["Stationary model",cc-1]=BFtable["Stationary model",cc-1]+1
	}
	}
	
	windows()
	par(mar=c(7.1,4.1,4.1,2.1))
	xpos=barplot(BFtable, beside=T,legend=T, ylim=c(0,25),
	ylab="Number of participants", xlab="", col=c("black", "white"))	
	
	#add x axis
	mtext(side=1, "Evidence", line=5)
	xlab=colMeans(xpos)
	labBF=c("anecdotal", "substantial", "strong", "very strong", "decisive")
	par(xpd=T)
	text(x=xlab,y=-2.5, labBF)
	savePlot(file="BFtable.ModelComparisonLinearIndividual.jpg", type="jpg")
	savePlot(file="BFtable.ModelComparisonLinearIndividual.eps", type="eps")


	

#BUGS parameters (used throughout the script)
	nBurnin    =5000
    chainLength=10000
	thinSteps  =10
	nChains    =4
    

#data for BUGS model (used throughout the script)
  dataList = list(
			    nVpn  = nVpn,
				nStay = t(nStayMatrix[1:8,]), 
				nTotal= t(nMatrix[1:8,])
				)

				
				
#================================
#estimate group-level drift model
BUGSmodel="groupLevelDrift.txt"


				
  #initialize starting values for the chains
  initsFunc <-function () {
    list (
	   groupSlopeM      = 0, 
	   groupSlopeSD     = 1,
	   groupInterceptM  = 0, 
	   groupInterceptSD = 1,
	   r				= 0,
	   indPhi			= cbind(rep(0, nVpn), rep(0,nVpn))
       )
  }
 
  parameters=c("intercept", "slope", "groupSlopeM", "indPhi",
			   "groupSlopeSD","groupInterceptM","groupInterceptSD", "r")

   			
  #send it off to BUGS
	samplesDrift=runBUGS(model=BUGSmodel,dataList, initsFunc, parameters,nBurnin, chainLength, thinSteps,nChains) 
	mcmcChains = as.matrix(samplesDrift$mcmc)

		
	#plot results
	nSamples=dim(mcmcChains)[1]
	slopeMatrix=matrix(NA, nrow=nVpn, ncol=nSamples)
	interceptMatrix=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	phiIntMatrix=matrix(NA, nrow=nVpn,ncol=nSamples)
	phiSlopeMatrix=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	
	windows()
	for (vpn in 1:nVpn) {
	
		slopeName=paste("slope[",vpn,"]", sep="")
		interceptName=paste("intercept[",vpn,"]", sep="")

	    slopeMatrix[vpn,]=mcmcChains[,slopeName]
		interceptMatrix[vpn,]=mcmcChains[,interceptName]
		
		phiIntMatrix[vpn,]=mcmcChains[,paste("indPhi[",vpn,",",1,"]",sep="")]
		phiSlopeMatrix[vpn,]=mcmcChains[,paste("indPhi[",vpn,",",2,"]",sep="")]
		
		plot(phiSlopeMatrix[vpn,],phiIntMatrix[vpn,], xlim=c(-6,6), ylim=c(-1.6,1.6))
		abline(v=0, lty="dashed")
		phi.axis=-6:6
		axis01=pnorm(phi.axis)
		axisBeta=1/7*axis01
		axisLab=round(axisBeta,3)
		axis(3, at=phi.axis, labels=axisLab, cex.axis=.7 )
		#par(ask=T)
	}
	
	
	#extract group-level Intercept
	groupInterceptProbitM.Drift=mcmcChains[,"groupInterceptM"]
	groupInterceptM.Drift=pnorm(groupInterceptProbitM.Drift)
		
	groupInterceptSD=mcmcChains[,"groupInterceptSD"]

	windows()
	layout(matrix(1:3, nrow=1))
	hist(groupInterceptM.Drift, col="blue")
	hist(groupInterceptSD, col="green")
	plot(groupInterceptM.Drift, groupInterceptSD)
	
	#extract group-level Slope
	groupSlope01ProbitM=mcmcChains[,"groupSlopeM"] 
	groupSlope01M.Drift=pnorm(groupSlope01ProbitM) #transform to slope scale
	groupSlopeM.Drift=(-1/7)*groupInterceptM.Drift+1/7*groupSlope01M.Drift
	
	groupSlopeProbitSD=mcmcChains[,"groupSlopeSD"]
	
	windows()
	layout(matrix(1:3, nrow=1))
	hist(groupSlopeM.Drift, col="blue")
	hist(groupSlopeProbitSD, col="green")
	plot(groupSlopeM.Drift, groupSlopeProbitSD)
	
	windows()
	useThis=sample(1:nSamples, 3000)
	plot(groupSlopeM.Drift[useThis],groupInterceptM.Drift[useThis])
	groupCorr=cor(groupSlopeM.Drift,groupInterceptM.Drift)
	mtext(paste("r =", round(groupCorr,2)))
	abline(lm(groupInterceptM.Drift~groupSlopeM.Drift), col="red")
	
	#plot individual parameters more "nicely" using 95%HDI
	plotIndividual = function(m, xlim=NULL,linepos=0,pch=20,...) {
	
	#plot individual parameter estimates from a matrix m (rows = vpn )
		source("HPD.r")
		
		meanPar=apply(m,1,mean)
		hpd95=apply(m,1,HPD)
		
		if(is.null(xlim)) xlim=range(hpd95)
		plot(meanPar,1:nVpn, xlim=xlim, yaxt="n",pch=pch, 
			 ylab="Participants",cex=1,...)
		abline(v=linepos, lty="dotted", col="black")
		points(meanPar,1:nVpn,pch=pch,cex=1)
		arrows(hpd95[1,],1:nVpn, hpd95[2,],1:nVpn, code=3, angle=90, length=.03)
		axis(2,1:nVpn, cex.axis=.4, labels=F)
	}	
	
	#plot group-level data
	plotGroupData = function (mInd,groupVecM,groupVecSD, xlab="Param", mainText="Parameter Estimate", xlim=c(-1/7,1/7), linepos=0, ymax=NULL,...) {
	
	windows(7,8)
	#plot individual data
	par(fig=c(0,1,0,0.7),mar=c(4.1,4.1,0.1,2.1))
	plotIndividual(mInd, main="",xlab=xlab, xlim=xlim, linepos=linepos,...)
		
	
	par(fig=c(0,1,0.7,1),mar=c(0.1,4.1,4.1,2.1), new=TRUE)
	dens=density(groupVecM)
	if(is.null(ymax)) ymax=1.1*max(dens$y)
	plot(NULL, xlim=xlim, ylim=c(0,ymax), xaxt="n",yaxt="n",ylab="",xlab="" , bty="n")
	abline(h=0)	
	abline(v=linepos,lty="dotted", col="black")
	hist(groupVecM,prob=T, add=T, col="lightgrey", border="lightgrey", breaks=40)
	lines(density(groupVecM), lwd=1)
	
	
	#add 95ci for group-level distribution
	source("HPD.r")
	hpd95=HPD(groupVecM)
	arrows(hpd95[1],ymax, hpd95[2],ymax, code=3, angle=90, length=.03, lwd=2)
	
	#add text
	meanMu=round(mean(groupVecM),2)
	text(x=meanMu, y= 1.05*ymax, bquote(mu==.(meanMu)), cex=.9, adj=c(.5,0),xpd=T)	
	mtext(mainText, line=2)
	return(round(hpd95,2))
	}
	
	pchInd=rep(1, times=nVpn)
	critBF=1
	meanSlope=apply(slopeMatrix,1,mean)
	pchInd[(BF>critBF) & (meanSlope<0)]=17
	pchInd[(BF>critBF) & (meanSlope>=0)]=15

	
	#Plot beta	
	sortierung=order(apply(slopeMatrix,1,mean))
	plotGroupData(mInd=slopeMatrix[sortierung,],groupVecM=groupSlopeM.Drift,linepos=0,pch=pchInd[sortierung],
				  groupVecSD=groupSlopeSD, xlab=expression(beta), mainText="")#expression(paste("Estimate of ",beta,"-parameter (slope)", sep="")))
	savePlot(file="slopeEstimate.DriftModel.jpg", type="jpg")
	savePlot(file="slopeEstimate.DriftModel.eps", type="eps")

	#Plot alpha
	sortierung=order(apply(interceptMatrix,1,mean))
	plotGroupData(mInd=interceptMatrix[sortierung,],groupVecM=groupInterceptM.Drift,linepos=.5,pch=pchInd[sortierung],
				  xlim=c(0,1), groupVecSD=groupInterceptSD, xlab=expression(alpha),
				  mainText="")#expression(paste("Estimate of ", alpha,"-parameter (intercept)", sep="")))
	savePlot(file="interceptEstimate.DriftModel.jpg", type="jpg")
	savePlot(file="interceptEstimate.DriftModel.eps", type="eps")


	#---------
	#calculate correlation between alpha and beta
	

	#obtain group-level distribution 
	library(mnormt)
 
	mInt   = mcmcChains[,"groupInterceptM"]
	sdInt  = mcmcChains[,"groupInterceptSD"]
	varInt = sdInt^2
	
	mSlope  = mcmcChains[,"groupSlopeM"]
	sdSlope = mcmcChains[,"groupSlopeSD"]
	varSlope= sdSlope^2
	
	r=mcmcChains[,"r"]
	covar=r*sdInt*sdSlope
	
	#sample from group-level distribution
	groupSamplePhi=matrix(NA, nrow=nSamples, ncol=2)
	for (s in 1:nSamples) {
		varcov=rbind(c(varInt[s], covar[s]),
					c(covar[s], varSlope[s]))
		groupSamplePhi[s,]=rmnorm(n=1, mean=c(mInt[s], mSlope[s]), varcov)
	}
	
	sampleIntPhi  =groupSamplePhi[,1]
	sampleSlopePhi=groupSamplePhi[,2]
	
	
	#plot group-level distribution on probit scale
	windows(10,5)
	layout(matrix(1:2, nrow=1))
	plot(sampleIntPhi, sampleSlopePhi, main="Group-level on probit scale")
	mtext(paste("r =", round(cor(sampleIntPhi, sampleSlopePhi),2)))
		

	#transform values to individual scale
	sampleInt=pnorm(sampleIntPhi) 
	sampleSlope01=pnorm(sampleSlopePhi) 
	sampleSlope=(-1/7)*sampleInt+1/7*sampleSlope01

	plot(sampleInt, sampleSlope, main="Group-level on individual scale")
	mtext(paste("r =", round(cor(sampleInt, sampleSlope),2)))
	
	
	
	#Plot alpha and beta parameters in one plot

	windows()

    #setup plot
	plot(NULL, xlim=c(-1/7, 1/7), xlab=expression(beta), ylim=c(0,1), ylab=expression(alpha), bty="n")
	lines(x=c(0,0),y=c(0,1), lty="dashed", col="black")
	#lines(x=c(-.07, .07), y=c(.5,.5), lty="dashed", col="red")
	lines(x=c(-1/7, 1/7), y=c(.5,.5), lty="dashed", col="black")
	lines(x=c(-1/7,0), y=c(1,0), lwd=1.5)
	lines(x=c(1/7,0), y=c(0,1), lwd=1.5)
	lines(x=c(1/7,0), y=c(0,0), lwd=1.5)
	lines(x=c(0,-1/7), y=c(1,1), lwd=1.5)
	
	
	#calculate mean slope and intercept
	
	#mean calculated on individual scale (not preferred because the distribution may be skewed)
		#slope
		bb=apply(slopeMatrix,1,mean)
		bb.hpd95=apply(slopeMatrix,1,HPD)

		#intercept
		aa=apply(interceptMatrix,1,mean)
		aa.hpd95=apply(interceptMatrix,1,HPD)
	
	#mean calculated on probit scale and then re-scaled (used in the paper)
		
		#intercept
		aaProbit=apply(phiIntMatrix,1,mean)
		aaX=pnorm(aaProbit)
		
		phi.aaX.hpd95=apply(phiIntMatrix,1,HPD)
		aaX.hpd95=pnorm(phi.aaX.hpd95)
	
		#slope
		bbProbit=apply(phiSlopeMatrix,1,mean)
		bb01X=pnorm(bbProbit)
		bbX=(-1/7)*aaX+1/7*bb01X
		
		bbX.hpd95Phi=apply(phiSlopeMatrix,1,HPD)
		bbX01.hpd95=pnorm(bbX.hpd95Phi)
		bbX.hpd95=(-1/7)*aaX.hpd95+1/7*bbX01.hpd95
		
	
	pchInd=rep(1, times=nVpn)
	critBF=1
	meanSlope=apply(slopeMatrix,1,mean)
	pchInd[(BF>critBF) & (meanSlope<0)]=17
	pchInd[(BF>critBF) & (meanSlope>=0)]=15

		
	#add points
	#points(bb,aa, pch=pchInd, cex=1) #mean based on individual scale
	points(bbX,aaX, pch=pchInd, cex=1) #mean based on probit scale

	#add correlation (calculated from the joint distribution on the group-level )
	rr=round(cor(sampleInt, sampleSlope),2)
	thisText=bquote(paste(italic(r)(alpha,beta) ==.(rr)))
	text(x=.1, y=.9,thisText)
	
	savePlot(file="jointParameterPlot.Driftmodel.jpg", type="jpg")
	savePlot(file="jointParameterPlot.Driftmodel.eps", type="eps")
	
	#indicate beta <0 and alpha >.5 (upper left quadrant in the plot)
	sum((aaX>.5)&(bbX<0))/39

	#identify cases where positive recency eventually leads to negative recency
	cond1=aaX>.5 #initially positive recency
	cond2=(aaX+bbX*7)<.5 #finally negative recency
	sum(cond1&cond2) 
	sum((!cond1)&(!cond2)) #no cases where negative recency turns into positive recency
	
	
	
#Posterior predictive
	windows()
	plot(NULL, xlim=c(0,7), ylim=c(0,1), xlab=expression(italic(r)), ylab=expression(pi), 
	     main="")#"Probability to stay on symbol depending on run length")
	abline(h=.5, lty="dashed", col="black", lwd=2)

	xx=0:7
	for(vpn in 1:nVpn) {
	
	thetaMatrix=matrix(NA,nrow=8, ncol=nSamples)
	for(sLength in 1:8) {
		thetaMatrix[sLength,]<-interceptMatrix[vpn,] + slopeMatrix[vpn,] * (sLength-1) 
	}	
	#if(mSlope[vpn]<0) linecolor="red" else linecolor="green"
	linecolor="darkgrey" #rainbow(nVpn)[vpn]
	yy=rowMeans(thetaMatrix)
	lines(xx,yy, col=linecolor)
	}
	
	#group prediction
	postPredMatrix=matrix(NA,nrow=8, ncol=nSamples)
	hpdGroupPostPred=matrix(NA, nrow=8, ncol=2)
	for(sLength in 1:8) {
		postPredMatrix[sLength,]=groupInterceptM.Drift+groupSlopeM.Drift*(sLength-1)
		hpdGroupPostPred[sLength,]=HPD(postPredMatrix[sLength,])
	}
	
	
	#add mean regression line
	xpos1=xx-.08
	xpos2=xx+.08
	yyBig=rowMeans(postPredMatrix)
	points(xpos1,yyBig, cex=2, pch=20)
	lines(xpos1,yyBig, lwd=3)
	
	#add error bars to the mean regression line
	arrows(xpos1,hpdGroupPostPred[,1], xpos1,hpdGroupPostPred[,2], code=3, angle=90, length=.03, lwd=1)

	#add observed means
	arrows(xpos2, ci95Table[1,], xpos2, ci95Table[2,], code = 3,angle=90, length=.05)
	points(xpos2,meanMatrix, cex=1, pch=17, col="white")
	points(xpos2,meanMatrix, cex=1, pch=2)
	

	savePlot(file="posteriorPredictive.Driftmodel.jpg", type="jpg")
    savePlot(file="posteriorPredictive.Driftmodel.eps", type="eps")


	
	
#================================================	
#estimate regression on theta separately for each individual
BUGSmodel="individualDrift.txt"

 			
  #initialize starting values for the chains
  initsFunc <-function () {
     list (
	   slope01Phi   = runif(nVpn,-1,1), 
	   interceptPhi = runif(nVpn, -1,1)
       )
  }
  
  parameters=c("intercept", "slope", "slope01Phi","interceptPhi" )

  #send it off to BUGS
	samplesDriftInd=runBUGS(model=BUGSmodel,dataList, initsFunc, parameters,nBurnin, chainLength, thinSteps,nChains) 
	mcmcChainsInd = as.matrix( samplesDriftInd$mcmc)
	
	
	#plot results
	nSamples=dim(mcmcChainsInd)[1]
	slopeMatrixInd=matrix(NA, nrow=nVpn, ncol=nSamples)
	interceptMatrixInd=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	for (vpn in 1:nVpn) {
	
		slopeName=paste("slope[",vpn,"]", sep="")
		interceptName=paste("intercept[",vpn,"]", sep="")

	    slopeMatrixInd[vpn,]=mcmcChainsInd[,slopeName]
		interceptMatrixInd[vpn,]=mcmcChainsInd[,interceptName]
	}
	
	windows()
	for(vpn in 1:nVpn) {
		slopeName=paste("slope[",vpn,"]", sep="")
		interceptName=paste("intercept[",vpn,"]", sep="")

		plot(mcmcChainsInd[,slopeName],mcmcChainsInd[,interceptName])
		abline(h=.5)
		abline(v=0)
		#par(ask=T)
	}
	
	
		
	#plot group-level data together with independent individual estimates 
	plotGroupData2 = function (mInd,mInd2,groupVecM,groupVecSD, xlab="Param", mainText="Parameter Estimate", xlim=c(-1/7,1/7), linepos=0) {
	
		windows(8,10)
		
		#plot individual data (estimated from hierarchical model)
		par(fig=c(0,1,0,0.7),mar=c(4.1,4.1,0.1,2.1))
		plotIndividual(mInd, main="",xlab=xlab, xlim=xlim, linepos=linepos)
		
		#plot indiviual-indiviudal data (estimated seperately)
		meanPar=apply(mInd2,1,mean)
		hpd95=apply(mInd2,1,HPD)
		ypos=(1:nVpn)#+.2
		points(meanPar,ypos,cex=.6, pch=2)
		#arrows(hpd95[1,],ypos, hpd95[2,],ypos, code=3, angle=90, length=.03)

			
		#add group-level 
		par(fig=c(0,1,0.7,1),mar=c(0.1,4.1,4.1,2.1), new=TRUE)
		dens=density(groupVecM)
		ymax=1.1*max(dens$y)
		plot(NULL, xlim=xlim, ylim=c(0,ymax), xaxt="n",yaxt="n",ylab="",xlab="" , bty="n")
		abline(h=0)	
		abline(v=linepos,lty="dotted", col="red")
		hist(groupVecM,prob=T, add=T, col="lightgrey", border="lightgrey", breaks=40)
		lines(density(groupVecM), lwd=1)
		

		#add 95ci for group-level distribution
		source("HPD.r")
		hpd95=HPD(groupVecM)
		arrows(hpd95[1],ymax, hpd95[2],ymax, code=3, angle=90, length=.03, lwd=2)
		
		#add text
		meanMu=round(mean(groupVecM),2)
		text(x=meanMu, y= 1.05*ymax, bquote(mu==.(meanMu)), cex=.7, adj=c(.5,0),xpd=T)	
		mtext(mainText, line=2)

	}
	
	
	#Slope
	sortierung=order(rowMeans(slopeMatrixInd))
	plotGroupData2(mInd=slopeMatrix[sortierung,],mInd2=slopeMatrixInd[sortierung,],groupVecM=groupSlopeM.Drift, 
				   groupVecSD=groupSlopeSD, xlab="Slope", mainText="")
	
	savePlot(file="slopeEstimateIndividual.jpg", type="jpg")
	savePlot(file="slopeEstimateIndividual.eps", type="eps")

	#Intercept
	sortierung=order(rowMeans(interceptMatrixInd))
	plotGroupData2(mInd=interceptMatrix[sortierung,],mInd2=interceptMatrixInd[sortierung,],groupVecM=groupInterceptM.Drift, 
				   groupVecSD=groupInterceptSD, xlab="Intercept", mainText="", xlim=c(0,1), linepos=.5)
	
	savePlot(file="slopeEstimateIndividual.jpg", type="jpg")
	savePlot(file="slopeEstimateIndividual.eps", type="eps")

	
#Posterior predictive on individual level
	windows(12,8)
	layout(matrix(1:10, nrow=2, byrow=T) )
	
	#draw regression line for each individual
	mSlope=apply(slopeMatrixInd,1,mean)
	mIntercept=apply(interceptMatrixInd,1,mean)
	xx=1:8
	
	for(vpn in 1:nVpn) {
		
		
		plot(NULL, xlim=c(1,8), ylim=c(0,1), xlab="Sequence Length", ylab="p(stay)", main="Probability to Stay on Symbol")
		abline(h=.5, lty="dashed", col="red", lwd=1.5)
		theta=mIntercept[vpn]+mSlope[vpn]*(xx-1)
		yy=theta
		points(xx,yy,type="b" , pch=2)
		
		#add 95% hdi
		thetaMatrix=matrix(NA, nrow=8, ncol=dim(slopeMatrixInd)[2])
		for(thisx in 1:8) thetaMatrix[thisx,]=interceptMatrixInd[vpn,]+slopeMatrixInd[vpn,]*(thisx-1)
		hpd95=apply(thetaMatrix,1,HPD)
		arrows(xx,hpd95[1,], xx,hpd95[2,], code=3, angle=90, length=.03, lwd=1)

		#add empirical data
		points(xx,pStayMatrix[,vpn],pch=16, cex=1.2)
		
		if((vpn%%10)==0) {
			savePlot(file=paste("PostPredictiveIndividual_",vpn,".jpg"), type="jpg")
			windows(12,8)
			layout(matrix(1:10, nrow=2, byrow=T) )
			}
	
		}
		savePlot(file=paste("PostPredictiveIndividual_",vpn,".jpg"), type="jpg")
	


#=============================
# Estimate stationary model 
BUGSmodel="groupLevelStationary.txt"

			
  #initialize starting values for the chains
  initsFunc <-function () {
    list (
	   interceptProbit  = runif(nVpn, -2,2),
	   groupInterceptM  = runif(1,-2,2), 
	   groupInterceptSD = runif(1,.1,3)
       )
  }
  
  parameters=c("theta", "intercept", "groupInterceptM","groupInterceptSD")

  #send it off to BUGS
	samplesStationary=runBUGS(model=BUGSmodel,dataList, initsFunc, parameters,nBurnin, chainLength, thinSteps,nChains) 
	mcmcChainsSimple = as.matrix( samplesStationary$mcmc)
	

	#plot results
	nSamples=dim(mcmcChainsSimple)[1]
	interceptMatrix=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	for (vpn in 1:nVpn) {
		interceptName=paste("intercept[",vpn,"]", sep="")
		interceptMatrix[vpn,]=mcmcChainsSimple[,interceptName]
	}
	
	#group-level intercept
	groupInterceptM.Stationary=mcmcChainsSimple[,"groupInterceptM"]
	groupInterceptSD=mcmcChainsSimple[,"groupInterceptSD"]
	
	windows()
	layout(matrix(1:3, nrow=1))
	hist(groupInterceptM.Stationary, col="blue")
	hist(groupInterceptSD, col="green")
	plot(groupInterceptM.Stationary, groupInterceptSD)
	

			
	#---------------------------
	#Savage-Dickey Density Ratio to obtain comparison against NULL-Model

	nSamples=length(interceptMatrix[vpn,])
	minBF=1/nSamples
	BF.simple=NULL
	for (vpn in 1:nVpn) {
	 densEst=density(interceptMatrix[vpn,], from=(1/512), to=1,n =512)
	 sdRatio=densEst$y[densEst$x==.5]
	 if(sdRatio<(minBF)) sdRatio=minBF
	 if(sdRatio>(1/minBF)) sdRatio=1/minBF
	 BF.simple[vpn]=sdRatio #values >1 indicate evidence for null model (p=.5) over simple model 
	}

	#define which model was better
	critBF=30 #100 = extreme evidence 
	nullModelLooses=1/BF.simple>critBF
	negRecency=rowMeans(interceptMatrix)<.5
	posRecency=rowMeans(interceptMatrix)>.5
	
	
	#plot individual data together with group-level data
	windows(7,8)
	sortierung=order(rowMeans(interceptMatrix))
	mInd=interceptMatrix[sortierung,]
	groupVecM=pnorm(groupInterceptM.Stationary)
	
	
	#define pch depending on BF
	pchInd=rep(1, times=nVpn)
	pchInd[negRecency & nullModelLooses]=17
	pchInd[posRecency & nullModelLooses]=15
	pchInd=pchInd[sortierung]
	
	par(fig=c(0,1,0,0.7),mar=c(4.1,4.1,0.1,2.1))
	plotIndividual(mInd, main="",xlab=expression(alpha), xlim=c(0,1),linepos=.5, pch=pchInd)
		
	par(fig=c(0,1,0.7,1),mar=c(0.1,4.1,4.1,2.1), new=TRUE)
	dens=density(groupVecM)
	ymax=1.1*max(dens$y)
	plot(NULL, xlim=c(0,1), ylim=c(0,ymax), xaxt="n",yaxt="n",ylab="",xlab="" , bty="n")
	abline(h=0)	
	abline(v=.5,lty="dotted", col="black")
	hist(groupVecM,prob=T, add=T, col="lightgrey", border="lightgrey", breaks=40)
	lines(density(groupVecM), lwd=1)
	
	
	#add 95ci for group-level distribution
	source("HPD.r")
	hpd95=HPD(groupVecM)
	arrows(hpd95[1],ymax, hpd95[2],ymax, code=3, angle=90, length=.03, lwd=2)
	
	#add text
	meanMu=round(mean(groupVecM),2)
	text(x=meanMu, y= 1.05*ymax, bquote(mu==.(meanMu)), cex=.9, adj=c(.5,0),xpd=T)	
	#mtext(expression(paste("Estimate for ",alpha,"-parameter - stationary model", sep="")), line=2)

	savePlot(file="interceptEstimateSimpleModel.StationaryModel.jpg", type="jpg")
	savePlot(file="interceptEstimateSimpleModel.StationaryModel.eps", type="eps")

	

#===============================
# Run individual drift model with the actual sequences as choices 
# as a manipulation check

	#get nStay for the actual sequence (instead of the empirical choices
	
	#Note: For a "perfect" predictor who always gets it right, 
	#pStay will be 0 for run length = 1 and 1 for all other run length
		
	pStayMatrixCheck=rbind(rep(0,nVpn),matrix(1,nrow=7, ncol=nVpn))
	nStayMatrixCheck=pStayMatrixCheck*nMatrix
	
 
  #replace nStay matrix in the dataList
  dataListCheck = dataList
  dataListCheck$nStay = t(nStayMatrixCheck[1:8,])

  BUGSmodel="individualDrift.txt"
 			
  #initialize starting values for the chains
  initsFunc <-function () {
     list (
	   slope01Phi   = runif(nVpn,-1,1), 
	   interceptPhi = runif(nVpn, -1,1)
       )
  }
  
  parameters=c("intercept", "slope", "slope01Phi","interceptPhi" )

  #send it off to BUGS
	samplesDriftIndCheck=runBUGS(model=BUGSmodel,dataListCheck, initsFunc, parameters,nBurnin, chainLength, thinSteps,nChains) 
	mcmcChainsCheck = as.matrix( samplesDriftIndCheck$mcmc)
	
	
	#plot results
	nSamples=dim(mcmcChainsCheck)[1]
	slopeMatrixIndCheck=matrix(NA, nrow=nVpn, ncol=nSamples)
	interceptMatrixIndCheck=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	slopePhiMatrixIndCheck=matrix(NA, nrow=nVpn, ncol=nSamples)
	interceptPhiMatrixIndCheck=matrix(NA, nrow=nVpn,ncol=nSamples)
	
	for (vpn in 1:nVpn) {
	
		slopeName=paste("slope[",vpn,"]", sep="")
		interceptName=paste("intercept[",vpn,"]", sep="")

	    slopeMatrixIndCheck[vpn,]=mcmcChainsCheck[,slopeName]
		interceptMatrixIndCheck[vpn,]=mcmcChainsCheck[,interceptName]
		
		#phi scale
		slopeNamePhi=paste("slope01Phi[",vpn,"]", sep="")
		interceptNamePhi=paste("interceptPhi[",vpn,"]", sep="")
		
		slopePhiMatrixIndCheck[vpn,]=mcmcChainsCheck[,slopeNamePhi]
		interceptPhiMatrixIndCheck[vpn,]=mcmcChainsCheck[,interceptNamePhi]
		
	}
	
	windows()
	layout(matrix(1:2,nrow=1))
	boxplot(t(slopeMatrixIndCheck), ylim=c(-1/7,1/7),col="palegoldenrod", outline=F, xlab="vpn", ylab="slope", main="Individual Slope Estimates - Check")
	abline(h=0, lty="dotted", col="red")
	boxplot(t(interceptMatrixIndCheck),ylim=c(0,1), col="palegoldenrod", outline=F, xlab="vpn", ylab="intercept", main="Individual Intercept Estimates - Check")

	
	#Slope
	windows()
	layout(matrix(1:2, nrow=1))
	plotIndividual(m=slopeMatrixIndCheck[sortierung,], main="Beta - Check ",xlab=expression(beta), xlim=c(-1/7,1/7), linepos=0)
	abline(v=0, lty="dotted")
	
	#Intercept
	sortierung=order(rowMeans(interceptMatrixIndCheck))
	plotIndividual(m=interceptMatrixIndCheck[sortierung,], main="Alpha - Check ",xlab=expression(alpha), xlim=c(0,1), linepos=0)
	abline(v=.5, lty="dotted")
	
	savePlot(file="individualCheck.jpg", type="jpg")
	savePlot(file="individualCheck.eps", type="eps")




#==============================
# Analysis of subjective confidence

	#descriptive data
	meanStakeVec=sapply(rawDataList, function(i) mean(i$BetResponse))
	sdStakeVec  =sapply(rawDataList, function(i) sd(i$BetResponse))

	
	#individual correlation between stakes (normalized) and run length 
	corRunlength=rep(NA, times=nVpn)

	for(vpn in 1:nVpn){
	 stake=scale(rawDataList[[vpn]]$BetResponse[2:nChoices])
	 seqLength=seqMatrix[vpn,]
	
	 corRunlength[vpn]=cor(stake,seqLength)
	}

	#test if there is a correlation between run length and confidence  
	mean(corRunlength)
	bootstrap(corRunlength)
	
	#correlation with drift parameter (beta)
	mSlope=apply(slopeMatrixInd,1,mean)
	cor(abs(mSlope),corRunlength)
	corrBootstrap(abs(mSlope),corRunlength, nStraps=100000)
	
	
#save image to file	
save.image(paste("DataAnalysisImage.",Sys.Date(),".RData"))	
